@@ -21,6 +21,7 @@ class Receiver:
         try:
             # core
             self.COMPANY_ID: str = env["PARASUT_COMPANY_ID"]
+            self.PARASUT_REGISTRY: str = env["PARASUT_REGISTRY"]
             self.PARASUT_BASE_DIR: str = env["PARASUT_BASE_DIR"]
             self.PARASUT_CLI_TEXT_EDITOR: str = env["PARASUT_CLI_TEXT_EDITOR"]
             # server
@@ -34,6 +35,13 @@ class Receiver:
             self.E_DOC_BROKER_RUBY_V: str = env["PARASUT_E_DOC_BROKER_RUBY_V"]
             self.E_DOC_BROKER_RAILS_PORT: str = env["PARASUT_E_DOC_BROKER_RAILS_PORT"]
             self.E_DOC_BROKER_DIR: str = env["PARASUT_E_DOC_BROKER_DIR"]
+            # post-office
+            self.POST_OFFICE_RUBY_V: str = env["PARASUT_POST_OFFICE_RUBY_V"]
+            self.POST_OFFICE_RAILS_PORT: str = env["PARASUT_POST_OFFICE_RAILS_PORT"]
+            self.POST_OFFICE_DIR: str = env["PARASUT_POST_OFFICE_DIR"]
+            # ubl-validator
+            self.UBL_VALIDATOR_MAVEN_V: str = env["PARASUT_UBL_VALIDATOR_MAVEN_V"]
+            self.UBL_VALIDATOR_DIR: str = env["PARASUT_UBL_VALIDATOR_DIR"]
             # phoenix
             self.PHOENIX_NODE_V: str = env["PARASUT_PHOENIX_NODE_V"]
             self.PHOENIX_YARN_V: str = env["PARASUT_PHOENIX_YARN_V"]
@@ -75,13 +83,16 @@ class Receiver:
             shared_logic="shared-logic",
         )
         self._core_commands: Dict[str, str] = dict(
-            source_nvm="source ~/.nvm/nvm.sh",
-            source_yvm="source ~/.yvm/yvm.sh",
-            source_rvm="source ~/.rvm/scripts/rvm",
+            source_asdf="source ~/.asdf/asdf.sh",
+            ember_release="ember release",
+            npm_set_parasut_registry=f"npm config set registry {self.PARASUT_REGISTRY}",
+            npm_login="npm login",
+            npm_publish="npm publish",
+            npm_delete_registry="npm config delete registry",
         )
         self._server_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_ruby_version=f"rvm use {self.SERVER_RUBY_V}",
+            choose_ruby_version=f"asdf local ruby {self.SERVER_RUBY_V}",
             launch_sidekiq="bundle exec sidekiq",
             launch_rails="rails server",
             switch_to_phoenix=f"rails runner 'puts Company.find({self.COMPANY_ID}).update!(used_app: \"{self.PHOENIX_SWITCH_NAME}\")'",  # noqa: E501
@@ -91,93 +102,124 @@ class Receiver:
         )
         self._billing_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_ruby_version=f"rvm use {self.BILLING_RUBY_V}",
+            choose_ruby_version=f"asdf local ruby {self.BILLING_RUBY_V}",
             launch_sidekiq="bundle exec sidekiq",
             launch_rails=f"rails server -p {self.BILLING_RAILS_PORT}",
         )
         self._e_doc_broker_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_ruby_version=f"rvm use {self.E_DOC_BROKER_RUBY_V}",
+            choose_ruby_version=f"asdf local ruby {self.E_DOC_BROKER_RUBY_V}",
             launch_sidekiq="bundle exec sidekiq",
             launch_rails=f"rails server -p {self.E_DOC_BROKER_RAILS_PORT}",
         )
+        self._post_office_commands: Dict[str, str] = dict(
+            launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
+            choose_ruby_version=f"asdf local ruby {self.POST_OFFICE_RUBY_V}",
+            launch_sidekiq="bundle exec sidekiq",
+            launch_rails=f"rails server -p {self.POST_OFFICE_RAILS_PORT}",
+        )
+        self._ubl_validator_commands: Dict[str, str] = dict(
+            launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
+            choose_maven_version=f"asdf local maven {self.UBL_VALIDATOR_MAVEN_V}",
+            launch_spring_boot="mvn spring-boot:run",
+        )
         self._phoenix_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_yarn_version=f"yvm use {self.PHOENIX_YARN_V}",
-            choose_node_version=f"nvm use {self.PHOENIX_NODE_V}",
+            choose_yarn_version=f"asdf local yarn {self.PHOENIX_YARN_V}",
+            choose_node_version=f"asdf local nodejs {self.PHOENIX_NODE_V}",
             ember_serve="PROJECT_TARGET=phoenix ember s",
         )
         self._client_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_yarn_version=f"yvm use {self.CLIENT_YARN_V}",
-            choose_node_version=f"nvm use {self.CLIENT_NODE_V}",
+            choose_yarn_version=f"asdf local yarn {self.CLIENT_YARN_V}",
+            choose_node_version=f"asdf local nodejs {self.CLIENT_NODE_V}",
             ember_serve=f"./node_modules/ember-cli/bin/ember s --live-reload-port {self.CLIENT_EMBER_PORT}",  # noqa: E501
         )
         self._trinity_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_yarn_version=f"yvm use {self.TRINITY_YARN_V}",
-            choose_node_version=f"nvm use {self.TRINITY_NODE_V}",
+            choose_yarn_version=f"asdf local yarn {self.TRINITY_YARN_V}",
+            choose_node_version=f"asdf local nodejs {self.TRINITY_NODE_V}",
             ember_serve="ember s --live-reload-port 6505",
         )
         self._ui_library_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_yarn_version=f"yvm use {self.UI_LIBRARY_YARN_V}",
-            choose_node_version=f"nvm use {self.UI_LIBRARY_NODE_V}",
+            choose_yarn_version=f"asdf local yarn {self.UI_LIBRARY_YARN_V}",
+            choose_node_version=f"asdf local yarn {self.UI_LIBRARY_NODE_V}",
             ember_serve=f"PROJECT_TARGET=phoenix ember s --live-reload-port {self.UI_LIBRARY_EMBER_PORT}",  # noqa: E501
         )
         self._shared_logic_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
-            choose_yarn_version=f"yvm use {self.SHARED_LOGIC_YARN_V}",
-            choose_node_version=f"nvm use {self.SHARED_LOGIC_NODE_V}",
+            choose_yarn_version=f"asdf local yarn {self.SHARED_LOGIC_YARN_V}",
+            choose_node_version=f"asdf local nodejs {self.SHARED_LOGIC_NODE_V}",
             ember_serve=f"ember s --live-reload-port {self.SHARED_LOGIC_EMBER_PORT}",
         )
         # run tasks
         self._task_run_server = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._server_commands["choose_ruby_version"],
                 self._server_commands["launch_rails"],
             ]
         )
         self._task_run_server_sidekiq = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._server_commands["choose_ruby_version"],
                 self._server_commands["launch_sidekiq"],
             ]
         )
         self._task_run_billing = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._billing_commands["choose_ruby_version"],
                 self._billing_commands["launch_rails"],
             ]
         )
         self._task_run_billing_sidekiq = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._billing_commands["choose_ruby_version"],
                 self._billing_commands["launch_sidekiq"],
             ]
         )
         self._task_run_e_doc_broker = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._e_doc_broker_commands["choose_ruby_version"],
                 self._e_doc_broker_commands["launch_rails"],
             ]
         )
         self._task_run_e_doc_broker_sidekiq = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._e_doc_broker_commands["choose_ruby_version"],
                 self._e_doc_broker_commands["launch_sidekiq"],
             ]
         )
+        self._task_run_post_office = " && ".join(
+            [
+                self._core_commands["source_asdf"],
+                self._post_office_commands["choose_ruby_version"],
+                self._post_office_commands["launch_rails"],
+            ]
+        )
+        self._task_run_post_office_sidekiq = " && ".join(
+            [
+                self._core_commands["source_asdf"],
+                self._post_office_commands["choose_ruby_version"],
+                self._post_office_commands["launch_sidekiq"],
+            ]
+        )
+        self._task_run_ubl_validator = " && ".join(
+            [
+                self._core_commands["source_asdf"],
+                self._ubl_validator_commands["choose_maven_version"],
+                self._ubl_validator_commands["launch_spring_boot"],
+            ]
+        )
         self._task_run_phoenix = " && ".join(
             [
-                self._core_commands["source_yvm"],
-                self._core_commands["source_nvm"],
+                self._core_commands["source_asdf"],
                 self._phoenix_commands["choose_yarn_version"],
                 self._phoenix_commands["choose_node_version"],
                 self._phoenix_commands["ember_serve"],
@@ -185,17 +227,27 @@ class Receiver:
         )
         self._task_run_shared_logic = " && ".join(
             [
-                self._core_commands["source_yvm"],
-                self._core_commands["source_nvm"],
+                self._core_commands["source_asdf"],
                 self._shared_logic_commands["choose_yarn_version"],
                 self._shared_logic_commands["choose_node_version"],
                 self._shared_logic_commands["ember_serve"],
             ]
         )
+        self._task_release_shared_logic = " && ".join(
+            [
+                self._core_commands["source_asdf"],
+                self._shared_logic_commands["choose_yarn_version"],
+                self._shared_logic_commands["choose_node_version"],
+                self._core_commands["ember_release"],
+                self._core_commands["npm_set_parasut_registry"],
+                self._core_commands["npm_login"],
+                self._core_commands["npm_publish"],
+                self._core_commands["npm_delete_registry"],
+            ]
+        )
         self._task_run_trinity = " && ".join(
             [
-                self._core_commands["source_yvm"],
-                self._core_commands["source_nvm"],
+                self._core_commands["source_asdf"],
                 self._trinity_commands["choose_yarn_version"],
                 self._trinity_commands["choose_node_version"],
                 self._trinity_commands["ember_serve"],
@@ -203,17 +255,27 @@ class Receiver:
         )
         self._task_run_ui_library = " && ".join(
             [
-                self._core_commands["source_yvm"],
-                self._core_commands["source_nvm"],
+                self._core_commands["source_asdf"],
                 self._ui_library_commands["choose_yarn_version"],
                 self._ui_library_commands["choose_node_version"],
                 self._ui_library_commands["ember_serve"],
             ]
         )
+        self._task_release_ui_library = " && ".join(
+            [
+                self._core_commands["source_asdf"],
+                self._ui_library_commands["choose_yarn_version"],
+                self._ui_library_commands["choose_node_version"],
+                self._core_commands["ember_release"],
+                self._core_commands["npm_set_parasut_registry"],
+                self._core_commands["npm_login"],
+                self._core_commands["npm_publish"],
+                self._core_commands["npm_delete_registry"],
+            ]
+        )
         self._task_run_client = " && ".join(
             [
-                self._core_commands["source_yvm"],
-                self._core_commands["source_nvm"],
+                self._core_commands["source_asdf"],
                 self._client_commands["choose_yarn_version"],
                 self._client_commands["choose_node_version"],
                 self._client_commands["ember_serve"],
@@ -222,28 +284,28 @@ class Receiver:
         # switch tasks
         self._task_switch_frontend_to_phoenix = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._server_commands["choose_ruby_version"],
                 self._server_commands["switch_to_phoenix"],
             ]
         )
         self._task_switch_frontend_to_trinity = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._server_commands["choose_ruby_version"],
                 self._server_commands["switch_to_trinity"],
             ]
         )
         self._task_switch_addling_to_invoice = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._server_commands["choose_ruby_version"],
                 self._server_commands["switch_to_invoice"],
             ]
         )
         self._task_switch_addling_to_receipt = " && ".join(
             [
-                self._core_commands["source_rvm"],
+                self._core_commands["source_asdf"],
                 self._server_commands["choose_ruby_version"],
                 self._server_commands["switch_to_receipt"],
             ]
@@ -272,6 +334,14 @@ class Receiver:
                 self._run_process(
                     [self._task_run_e_doc_broker_sidekiq], show_output=True
                 )
+            elif "post-office" == repo_name:
+                self._run_process([self._task_run_post_office], show_output=True)
+            elif "post-office-sidekiq" == repo_name:
+                self._run_process(
+                    [self._task_run_post_office_sidekiq], show_output=True
+                )
+            elif "ubl-validator" == repo_name:
+                self._run_process([self._task_run_ubl_validator], show_output=True)
             elif "phoenix" == repo_name:
                 self._run_process([self._task_run_phoenix], show_output=True)
             elif "shared-logic" == repo_name:
@@ -287,6 +357,21 @@ class Receiver:
         except Exception as e:
             print(e)
 
+    def release_repo(self, repo_name: str) -> None:
+        base_path: str = self._find_repo_path(repo_name)
+
+        self._change_directory(base_path)
+
+        try:
+            if "shared-logic" == repo_name:
+                self._run_process([self._task_release_shared_logic], show_output=True)
+            elif "ui-library" == repo_name:
+                self._run_process([self._task_release_ui_library], show_output=True)
+        except KeyboardInterrupt:
+            pass
+        except Exception as e:
+            print(e)
+
     def create_parasut_ws_setup(self, repos: List[str]) -> None:
         session: Optional[Session]
 
@@ -316,6 +401,16 @@ class Receiver:
                     f"{self.PARASUT_BASE_DIR}/{self.E_DOC_BROKER_DIR}"
                 )
                 self._launch_parasut_e_doc_broker_repo()
+            elif "post-office" == repo_name:
+                self._change_directory(
+                    f"{self.PARASUT_BASE_DIR}/{self.POST_OFFICE_DIR}"
+                )
+                self._launch_parasut_post_office_repo()
+            elif "ubl-validator" == repo_name:
+                self._change_directory(
+                    f"{self.PARASUT_BASE_DIR}/{self.UBL_VALIDATOR_DIR}"
+                )
+                self._launch_parasut_ubl_validator_repo()
             elif "phoenix" == repo_name:
                 self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.PHOENIX_DIR}")
                 self._launch_parasut_phoenix_repo()
@@ -369,6 +464,16 @@ class Receiver:
                     f"{self.PARASUT_BASE_DIR}/{self.E_DOC_BROKER_DIR}"
                 )
                 self._launch_parasut_e_doc_broker_editor()
+            elif "post-office" == repo_name:
+                self._change_directory(
+                    f"{self.PARASUT_BASE_DIR}/{self.POST_OFFICE_DIR}"
+                )
+                self._launch_parasut_post_office_editor()
+            elif "ubl-validator" == repo_name:
+                self._change_directory(
+                    f"{self.PARASUT_BASE_DIR}/{self.UBL_VALIDATOR_DIR}"
+                )
+                self._launch_parasut_ubl_validator_editor()
             elif "phoenix" == repo_name:
                 self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.PHOENIX_DIR}")
                 self._launch_parasut_phoenix_editor()
@@ -655,6 +760,10 @@ class Receiver:
             return f"{self.PARASUT_BASE_DIR}/{self.BILLING_DIR}"
         elif repo_name == "e-doc-broker" or "e-doc-broker" in repo_name:
             return f"{self.PARASUT_BASE_DIR}/{self.E_DOC_BROKER_DIR}"
+        elif repo_name == "post-office" or "post-office" in repo_name:
+            return f"{self.PARASUT_BASE_DIR}/{self.POST_OFFICE_DIR}"
+        elif repo_name == "ubl-validator" or "ubl-validator" in repo_name:
+            return f"{self.PARASUT_BASE_DIR}/{self.UBL_VALIDATOR_DIR}"
         elif repo_name == "client":
             return f"{self.PARASUT_BASE_DIR}/{self.CLIENT_DIR}"
         elif repo_name == "phoenix":
@@ -747,6 +856,7 @@ class Receiver:
         server_pane.send_keys(
             " && ".join(
                 [
+                    self._server_commands["choose_ruby_version"],
                     self._server_commands["launch_text_editor"],
                 ]
             )
@@ -787,6 +897,7 @@ class Receiver:
         billing_pane.send_keys(
             " && ".join(
                 [
+                    self._billing_commands["choose_ruby_version"],
                     self._billing_commands["launch_text_editor"],
                 ]
             )
@@ -829,11 +940,83 @@ class Receiver:
         e_doc_broker_pane.send_keys(
             " && ".join(
                 [
+                    self._e_doc_broker_commands["choose_ruby_version"],
                     self._e_doc_broker_commands["launch_text_editor"],
                 ]
             )
         )
 
+    def _launch_parasut_post_office_repo(self) -> None:
+        post_office_window: Window = self._tmux_session_parasut_ws_setup.new_window(
+            attach=False, window_name="post_office"
+        )
+        post_office_pane: Pane = post_office_window.attached_pane
+        post_office_sidekiq_pane: Pane = post_office_window.split_window(vertical=False)
+
+        post_office_window.select_layout("tiled")
+        # panes
+        post_office_pane.send_keys(
+            " && ".join(
+                [
+                    self._post_office_commands["choose_ruby_version"],
+                    self._post_office_commands["launch_rails"],
+                ]
+            )
+        )
+        post_office_sidekiq_pane.send_keys(
+            " && ".join(
+                [
+                    self._post_office_commands["choose_ruby_version"],
+                    self._post_office_commands["launch_sidekiq"],
+                ]
+            )
+        )
+
+    def _launch_parasut_post_office_editor(self) -> None:
+        post_office_window: Window = self._tmux_session_parasut_ws_editor.new_window(
+            attach=False, window_name="post_office"
+        )
+        post_office_pane: Pane = post_office_window.attached_pane
+
+        post_office_pane.send_keys(
+            " && ".join(
+                [
+                    self._post_office_commands["choose_ruby_version"],
+                    self._post_office_commands["launch_text_editor"],
+                ]
+            )
+        )
+
+    def _launch_parasut_ubl_validator_repo(self) -> None:
+        ubl_validator_window: Window = self._tmux_session_parasut_ws_setup.new_window(
+            attach=False, window_name="ubl_validator"
+        )
+        ubl_validator_pane: Pane = ubl_validator_window.attached_pane
+
+        ubl_validator_pane.send_keys(
+            " && ".join(
+                [
+                    self._ubl_validator_commands["choose_maven_version"],
+                    self._ubl_validator_commands["launch_spring_boot"],
+                ]
+            )
+        )
+
+    def _launch_parasut_ubl_validator_editor(self) -> None:
+        ubl_validator_window: Window = self._tmux_session_parasut_ws_editor.new_window(
+            attach=False, window_name="ubl_validator"
+        )
+        ubl_validator_pane: Pane = ubl_validator_window.attached_pane
+
+        ubl_validator_pane.send_keys(
+            " && ".join(
+                [
+                    self._ubl_validator_commands["choose_maven_version"],
+                    self._ubl_validator_commands["launch_text_editor"],
+                ]
+            )
+        )
+
     def _launch_parasut_phoenix_repo(self) -> None:
         phoenix_window: Window = self._tmux_session_parasut_ws_setup.new_window(
             attach=False, window_name="phoenix"
@@ -843,8 +1026,8 @@ class Receiver:
         phoenix_pane.send_keys(
             " && ".join(
                 [
-                    self._phoenix_commands["choose_yarn_version"],
                     self._phoenix_commands["choose_node_version"],
+                    self._phoenix_commands["choose_yarn_version"],
                     self._phoenix_commands["ember_serve"],
                 ]
             )
@@ -859,6 +1042,8 @@ class Receiver:
         phoenix_pane.send_keys(
             " && ".join(
                 [
+                    self._phoenix_commands["choose_node_version"],
+                    self._phoenix_commands["choose_yarn_version"],
                     self._phoenix_commands["launch_text_editor"],
                 ]
             )
@@ -873,8 +1058,8 @@ class Receiver:
         client_pane.send_keys(
             " && ".join(
                 [
-                    self._client_commands["choose_yarn_version"],
                     self._client_commands["choose_node_version"],
+                    self._client_commands["choose_yarn_version"],
                     self._client_commands["ember_serve"],
                 ]
             )
@@ -889,6 +1074,8 @@ class Receiver:
         client_pane.send_keys(
             " && ".join(
                 [
+                    self._client_commands["choose_node_version"],
+                    self._client_commands["choose_yarn_version"],
                     self._client_commands["launch_text_editor"],
                 ]
             )
@@ -903,8 +1090,8 @@ class Receiver:
         trinity_pane.send_keys(
             " && ".join(
                 [
-                    self._trinity_commands["choose_yarn_version"],
                     self._trinity_commands["choose_node_version"],
+                    self._trinity_commands["choose_yarn_version"],
                     self._trinity_commands["ember_serve"],
                 ]
             )
@@ -919,6 +1106,8 @@ class Receiver:
         trinity_pane.send_keys(
             " && ".join(
                 [
+                    self._trinity_commands["choose_node_version"],
+                    self._trinity_commands["choose_yarn_version"],
                     self._trinity_commands["launch_text_editor"],
                 ]
             )
@@ -933,8 +1122,8 @@ class Receiver:
         ui_library_pane.send_keys(
             " && ".join(
                 [
-                    self._ui_library_commands["choose_yarn_version"],
                     self._ui_library_commands["choose_node_version"],
+                    self._ui_library_commands["choose_yarn_version"],
                     self._ui_library_commands["ember_serve"],
                 ]
             )
@@ -949,6 +1138,8 @@ class Receiver:
         ui_library_pane.send_keys(
             " && ".join(
                 [
+                    self._ui_library_commands["choose_node_version"],
+                    self._ui_library_commands["choose_yarn_version"],
                     self._ui_library_commands["launch_text_editor"],
                 ]
             )
@@ -963,8 +1154,8 @@ class Receiver:
         shared_logic_pane.send_keys(
             " && ".join(
                 [
-                    self._shared_logic_commands["choose_yarn_version"],
                     self._shared_logic_commands["choose_node_version"],
+                    self._shared_logic_commands["choose_yarn_version"],
                     self._shared_logic_commands["ember_serve"],
                 ]
             )
@@ -979,6 +1170,8 @@ class Receiver:
         shared_logic_pane.send_keys(
             " && ".join(
                 [
+                    self._shared_logic_commands["choose_node_version"],
+                    self._shared_logic_commands["choose_yarn_version"],
                     self._shared_logic_commands["launch_text_editor"],
                 ]
             )
